# -*- coding: utf-8 -*-
# Generated by Django 1.11.9 on 2018-01-24 15:14
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('jasonpi', '0001_initial'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='provider',
            unique_together=set([('uid', 'provider')]),
        ),
    ]
